@@ -11,7 +11,7 @@
 /* Try to use as much of the GNU C library as possible: */
 #define _GNU_SOURCE
 
-#include "src/compiled.h"          /* GAP headers                */
+#include "compiled.h" // GAP headers
 
 #if GAP_KERNEL_MAJOR_VERSION >= 6
 #include "src/profile.h"
@@ -27,6 +27,7 @@
 
 #include "pkgconfig.h"    /* our own autoconf results */
 
+#include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -190,7 +191,7 @@ static int ignoredpidslen;
 
 static int IO_CheckForIgnoredPid( int pid );
 
-static RETSIGTYPE (*oldhandler)(int whichsig) = 0;  /* the old handler */
+static void (*oldhandler)(int whichsig) = 0;  /* the old handler */
 
 static void IO_HandleChildSignal(int retcode, int status)
 {
@@ -216,7 +217,7 @@ static void IO_HandleChildSignal(int retcode, int status)
 }
 
 #ifdef HAVE_SIGNAL
-RETSIGTYPE IO_SIGCHLDHandler( int whichsig )
+void IO_SIGCHLDHandler( int whichsig )
 {
   int retcode,status;
   /* We collect information about our child processes that have
@@ -682,6 +683,23 @@ static Obj FuncIO_readlink(Obj self,Obj path,Obj buf,Obj bufsize)
 }
 #endif
 
+static Obj FuncIO_realpath(Obj self, Obj path)
+{
+    if (!IS_STRING_REP(path)) {
+        SyClearErrorNo();
+        return Fail;
+    }
+
+    char buf[PATH_MAX];
+    if (realpath(CSTR_STRING(path), buf)) {
+        return MakeImmString(buf);
+    }
+
+    SySetErrorNo();
+    return Fail;
+}
+
+
 static Obj FuncIO_chdir(Obj self,Obj pathname)
 {
   int res;
@@ -1395,21 +1413,18 @@ static Obj FuncIO_select(Obj self, Obj inlist, Obj outlist, Obj exclist,
   Obj o;
   time_t t1,t2;
 
-  while (inlist == (Obj) 0 || !(IS_PLIST(inlist)))
-    inlist = ErrorReturnObj(
+  if (!IS_PLIST(inlist))
+    ErrorMayQuit(
            "<inlist> must be a list of small integers (not a %s)",
-           (Int)TNAM_OBJ(inlist),0L,
-           "you can replace <inlist> via 'return <inlist>;'" );
-  while (outlist == (Obj) 0 || !(IS_PLIST(outlist)))
-    outlist = ErrorReturnObj(
+           (Int)TNAM_OBJ(inlist),0);
+  if (!IS_PLIST(outlist))
+    ErrorMayQuit(
            "<outlist> must be a list of small integers (not a %s)",
-           (Int)TNAM_OBJ(outlist),0L,
-           "you can replace <outlist> via 'return <outlist>;'" );
-  while (exclist == (Obj) 0 || !(IS_PLIST(exclist)))
-    exclist = ErrorReturnObj(
+           (Int)TNAM_OBJ(outlist),0);
+  if (!IS_PLIST(exclist))
+    ErrorMayQuit(
            "<exclist> must be a list of small integers (not a %s)",
-           (Int)TNAM_OBJ(exclist),0L,
-           "you can replace <exclist> via 'return <exclist>;'" );
+           (Int)TNAM_OBJ(exclist),0);
 
   FD_ZERO(&infds);
   FD_ZERO(&outfds);
@@ -1915,6 +1930,8 @@ static StructGVarFunc GVarFuncs [] = {
   GVAR_FUNC(IO_readlink, 3, "path, buf, bufsize"),
 #endif
 
+  GVAR_FUNC(IO_realpath, 1, "path"),
+
 #ifdef HAVE_MKDIR
   GVAR_FUNC(IO_mkdir, 2, "pathname, mode"),
 #endif
@@ -3015,6 +3032,6 @@ StructInitInfo * Init__io ( void )
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
